/* $Id$ */

#ifdef __cplusplus
extern "C" {
#endif
#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"
#ifdef __cplusplus
}
#endif
#include <time.h>
#define false (0)
#define true (!false)

#include <../../gsm-common.h>
#include <../../gsm-api.h>

typedef struct {
  int Empty;                                       /* Is this entry empty? */
  char Name[GSM_MAX_PHONEBOOK_NAME_LENGTH + 1];     /* Plus 1 for null terminator. */
  char Number[GSM_MAX_PHONEBOOK_NUMBER_LENGTH + 1]; /* Number */
  GSM_MemoryType MemoryType;                        /* Type of memory (SIM/internal) */
  int Group;                                        /* Group */
} Perl_GSM_PhonebookEntry;

typedef struct {
  int AlarmEnabled; /* Is the alarm set? */
  int Year;          /* The complete year specification - e.g. 1999. Y2K :-) */
  int Month;	     /* January = 1 */
  int Day;
  int Hour;
  int Minute;
  int Second;
  int Timezone;
} Perl_GSM_DateTime;

#define GSMDATE_TO_TM(GSMP,TM,HASH)\
{\
  time_t rt;\
\
  TM.tm_year = GSMP->Year;\
  if(TM.tm_year > 1900)\
    TM.tm_year -= 1900;\
  TM.tm_mon  = GSMP->Month - 1;\
  TM.tm_mday = GSMP->Day;\
  TM.tm_hour = GSMP->Hour;\
  TM.tm_min  = GSMP->Minute;\
  TM.tm_sec  = GSMP->Second;\
  TM.tm_isdst=-1;\
  TM.tm_wday = 0;\
  TM.tm_yday = 0;\
  rt = mktime(&TM);\
  hv_store(HASH,"Date",4,newSViv(rt),0);\
}

typedef struct {
  Perl_GSM_DateTime Time;	            /* Date of reception/response of messages. */
  int Length;                               /* Length of the SMS message. */
  int Validity;                             /* Validity Period of the SMS message. */
  char MessageText[GSM_MAX_SMS_LENGTH + 1]; /* Room for null term. */
  GSM_MessageCenter MessageCenter;          /* SMS Center. */
  char Sender[GSM_MAX_SENDER_LENGTH + 1];   /* Sender of the SMS message. */
  char Destination[GSM_MAX_DESTINATION_LENGTH + 1]; /* Destination of the message. */
  int MessageNumber;                        /* Location in the memory. */
  GSM_MemoryType MemoryType;                /* Type of memory message is stored in. */
  GSM_SMSMessageType Type;                  /* Type of the SMS message */
  GSM_SMSMessageStatus Status;              /* Status of the SMS message */
  int Class;                                /* Class Message: 0, 1, 2, 3 or none; see GSM 03.38 */
  int EightBit;                            /* Indicates whether SMS contains 8 bit data or not */
  int Compression;                         /* Indicates whether SMS contains compressed data or not */
} Perl_GSM_SMSMessage;


static int
not_here(s)
char *s;
{
    croak("%s not implemented on this architecture", s);
    return -1;
}
static double
constant(name, arg)
char *name;
int arg;
{
    errno = 0;
    switch (*name) {
    case 'A':
	break;
    case 'B':
	break;
    case 'C':
	break;
    case 'D':
	break;
    case 'E':
	break;
    case 'F':
	break;
    case 'G':
	if (strEQ(name, "GSM_GROUPS_NOT_SUPPORTED"))
#ifdef GSM_GROUPS_NOT_SUPPORTED
	    return GSM_GROUPS_NOT_SUPPORTED;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_DESTINATION_LENGHT"))
#ifdef GSM_MAX_DESTINATION_LENGHT
	    return GSM_MAX_DESTINATION_LENGHT;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_DEVICE_NAME_LENGTH"))
#ifdef GSM_MAX_DEVICE_NAME_LENGTH
	    return GSM_MAX_DEVICE_NAME_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_PHONEBOOK_NAME_LENGTH"))
#ifdef GSM_MAX_PHONEBOOK_NAME_LENGTH
	    return GSM_MAX_PHONEBOOK_NAME_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_PHONEBOOK_NUMBER_LENGTH"))
#ifdef GSM_MAX_PHONEBOOK_NUMBER_LENGTH
	    return GSM_MAX_PHONEBOOK_NUMBER_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_SENDER_LENGTH"))
#ifdef GSM_MAX_SENDER_LENGTH
	    return GSM_MAX_SENDER_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_SMS_CENTRE_LENGTH"))
#ifdef GSM_MAX_SMS_CENTRE_LENGTH
	    return GSM_MAX_SMS_CENTRE_LENGTH;
#else
	    goto not_there;
#endif
	if (strEQ(name, "GSM_MAX_SMS_LENGTH"))
#ifdef GSM_MAX_SMS_LENGTH
	    return GSM_MAX_SMS_LENGTH;
#else
	    goto not_there;
#endif
	break;
    case 'H':
	break;
    case 'I':
	break;
    case 'J':
	break;
    case 'K':
	break;
    case 'L':
	break;
    case 'M':
	break;
    case 'N':
	break;
    case 'O':
	break;
    case 'P':
	break;
    case 'Q':
	break;
    case 'R':
	break;
    case 'S':
	break;
    case 'T':
	break;
    case 'U':
	break;
    case 'V':
	break;
    case 'W':
	break;
    case 'X':
	break;
    case 'Y':
	break;
    case 'Z':
	break;
    }
    errno = EINVAL;
    return 0;

not_there:
    errno = ENOENT;
    return 0;
}

typedef HV HvObject;
typedef AV AvObject;

MODULE = Gnokii::GSM		PACKAGE = Gnokii::GSM

int
Initialize(ghash)
HvObject *ghash;
PREINIT:
int count=0;
char *model, *device;
CODE:
           RETVAL = GSM_Initialise(SvPV(*hv_fetch(ghash,"model",5,0),na),SvPV(*hv_fetch(ghash,"device",6,0),na),"default",SvIV(*hv_fetch(ghash,"connection",10,0)),0);
           while (count++ < 2000 && *GSM_LinkOK == 0)
             usleep(50000);

	   if (*GSM_LinkOK == 0) 
           {
	     fprintf (stderr, _("Hmmm... GSM_LinkOK never went true. Quitting. \n"));
	     exit(-1);
	   }
OUTPUT:
	   RETVAL

int
GetMemoryLocation(ghash,index,location)
HvObject *ghash;
int index;
char *location;
PREINIT:
GSM_Error error;
Perl_GSM_PhonebookEntry *entry;
SV **ssv;
CODE:
           entry = (Perl_GSM_PhonebookEntry *) malloc (sizeof(Perl_GSM_PhonebookEntry));
           ssv = hv_fetch((HV *)SvRV(*hv_fetch(ghash, "MEMORY_TYPES", 12, 0)), location, 2, 0);
           if(ssv != NULL)
           { 
	     memset(entry,0,sizeof(Perl_GSM_PhonebookEntry));
	     entry->MemoryType = SvIV(*ssv);
	     error = GSM->GetMemoryLocation(index, entry);
	     /*	     printf("error=%d location=%s index=%d entry->Name= %s entry->empty=%d\n",error,location,index,entry->Name,entry->Name,entry->Empty);*/
	     RETVAL = error;
	     if(error == GE_NONE)
	     {
	       char l[2];
	       HvObject *hentry;
	       
	       hentry=newHV();
	       l[0] = *location;
	       l[1] = '\0';
	       hentry = newHV();
	       hv_store(hentry, "Name", 4, newSVpv(entry->Name,0), 0);
	       hv_store(hentry, "Number", 6, newSVpv(entry->Number,0), 0);
	       hv_store(hentry, "Memory", 6, newSVpv(l,0), 0);
	       hv_store(hentry, "Group", 5, newSViv(entry->Group), 0);
	       hv_store(ghash, "PBEntry", 7, newRV_inc((SV *)hentry), 0);
	     }
	   }
           else
             RETVAL = GE_INVALIDMEMORYTYPE;
OUTPUT:
	   RETVAL

int
WritePhonebookLocation(ghash, hentry, location)
HvObject *ghash;
HV *hentry;
int location;
PREINIT:
Perl_GSM_PhonebookEntry *entry;
SV **ssv;
CODE:
        RETVAL=GE_NONE;
        entry = (Perl_GSM_PhonebookEntry *)malloc(sizeof(Perl_GSM_PhonebookEntry));
        strcpy(entry->Name, SvPV(*hv_fetch(hentry, "Name", 4, 0),na));
        strcpy(entry->Number, SvPV(*hv_fetch(hentry, "Number", 6, 0),na));

        ssv = hv_fetch((HV *)SvRV(*hv_fetch(ghash, "MEMORY_TYPES", 12, 0)), SvPV(*hv_fetch(hentry, "Memory", 6, 0),na), 2, 0);
        if(ssv != NULL)
        { 
	  entry->MemoryType = SvIV(*ssv);
	  entry->Group = SvIV(*hv_fetch(hentry, "Group",5 ,0));
	  RETVAL = GSM->WritePhonebookLocation(location,entry);
	}
        else
          RETVAL = GE_INVALIDMEMORYTYPE;
OUTPUT:
        RETVAL

int
GetMemStat(ghash, memtype)
HvObject *ghash;
int memtype;
PREINIT:
GSM_MemoryStatus status;
HvObject *stat_fetched;
CODE:
        stat_fetched = newHV();
	status.MemoryType=memtype;
        RETVAL = GSM->GetMemoryStatus(&status);
        if(RETVAL == GE_NONE)
        {
	  hv_store(stat_fetched, "Used", 4, newSViv(status.Used), 0);
	  hv_store(stat_fetched, "Free", 4, newSViv(status.Free), 0);
	}
OUTPUT:
	RETVAL

int
GetSMSStatus(ghash)
HvObject *ghash;
PREINIT:
GSM_SMSStatus status;
HvObject *smsstat;
CODE:
        RETVAL = GSM->GetSMSStatus(&status);
        if(RETVAL == GE_NONE)
        {
	  smsstat = newHV();
	  hv_store(smsstat, "Unread", 6, newSViv(status.UnRead),0);
	  hv_store(smsstat, "Number", 6, newSViv(status.Number),0);
	  hv_store(ghash, "SMSStatus", 9, newRV_inc((SV *) smsstat), 0);
	}
OUTPUT:
	RETVAL

int
GetSMSCenter(ghash, index)
HvObject *ghash;
int index;
PREINIT:
GSM_MessageCenter *center;
CODE:
        center = (GSM_MessageCenter *)malloc(sizeof(GSM_MessageCenter));
        center->No = index;
        RETVAL = GSM->GetSMSCenter(center);
        if(RETVAL == GE_NONE)
        {
	  HvObject *smsc;

	  smsc = newHV();
	  hv_store(smsc, "No", 2, newSViv(center->No), 0);
	  hv_store(smsc, "Name", 4, newSVpv(center->Name, 0), 0);
	  hv_store(smsc, "Number", 6, newSVpv(center->Number, 0), 0);
	  hv_store(ghash, "SMSCenter", 9, newRV_inc((SV *) smsc), 0);
	}
OUTPUT:
	RETVAL

int
GetSMSMessage(ghash, location, index)
HvObject *ghash;
char *location;
int index;
PREINIT:
GSM_MemoryType memory;
Perl_GSM_SMSMessage *msg;
HvObject *message;
struct tm t;
CODE:
/*        if(!strcasecmp(location,"A"))
	  memory = GMT_SM;
	  else*/
	  memory = GMT_ME;
        msg = (Perl_GSM_SMSMessage *)malloc(sizeof(Perl_GSM_SMSMessage));
	RETVAL = GSM->GetSMSMessage(index, msg);
	if(RETVAL == GE_NONE)
	{
	  Perl_GSM_DateTime *pgt;

	  pgt = &(msg->Time);
	  message = newHV();
	  GSMDATE_TO_TM(pgt,t,message);
	  hv_store(message,"Length", 6, newSViv(msg->Length),0);
	  hv_store(message,"Validity", 8, newSViv(msg->Validity),0);
	  hv_store(message,"MessageText", 11, newSVpv(msg->MessageText,0),0);
	  hv_store(message,"MessageCenter",13, newSVpv(msg->MessageCenter.Number,0),0);
	  hv_store(message,"Sender", 6, newSVpv(msg->Sender, 0),0);
	  hv_store(message,"Destination", 10, newSVpv(msg->Destination, 0),0);
	  hv_store(message,"Memory", 6, newSVpv(location, 0), 0);
	  hv_store(message,"Type", 4, newSViv(msg->Type),0);
	  hv_store(message,"Status", 6, newSViv(msg->Status),0);
	  hv_store(message,"Class", 5, newSViv(msg->Class),0);
	  hv_store(message,"EightBit", 8, newSViv(msg->EightBit),0);
	  hv_store(message,"Compression", 11, newSViv(msg->Compression),0);
	  hv_store(ghash, "Msg", 3, newRV_inc((SV *)message),0);
	}
        free(msg);
OUTPUT:
	RETVAL

int
DeleteSMSMessage(ghash, memory, location)
HvObject *ghash;
char *memory;
int location;
PREINIT:
Perl_GSM_SMSMessage *message;
GSM_MemoryType mtype;
CODE:
        if(!strcasecmp(memory,"A"))
	  mtype = GMT_SM;
	else
	  mtype = GMT_ME;
       message = (Perl_GSM_SMSMessage *)malloc(sizeof(Perl_GSM_SMSMessage));
       RETVAL = GSM->DeleteSMSMessage(location, message);
OUTPUT:
       RETVAL

int
SendSMSMessage(ghash, msg)
HvObject *ghash;
HV *msg;
PREINIT:
Perl_GSM_SMSMessage *message;
char *tmp;
CODE:  
       message = (Perl_GSM_SMSMessage *)malloc(sizeof(Perl_GSM_SMSMessage));
       message->Validity = 4320;
       strcpy(message->MessageText, SvPV(*hv_fetch(msg, "MessageText", 11, 0),na));
       strcpy(message->MessageCenter.Number, SvPV(*hv_fetch(msg, "MessageCentre", 13, 0), na));
       strcpy(message->Destination , SvPV(*hv_fetch(msg, "Destination", 11, 0), na));
       RETVAL = GSM->SendSMSMessage(message);
OUTPUT:
       RETVAL


int
GetRFLevel(ghash,unit)
HvObject *ghash;
int unit;
PREINIT:
float *f;
CODE:
        f = (float *)malloc(sizeof(float));
        RETVAL = GSM->GetRFLevel(&unit ,f);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "RFLevel", 7, newSVnv((double)*f),0);
OUTPUT:
	RETVAL

int
GetBatteryLevel(ghash,unit)
HvObject *ghash;
int unit;
PREINIT:
float *f;
CODE:
        f = (float *)malloc(sizeof(float));
        RETVAL = GSM->GetBatteryLevel(&unit, f);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "BatteryLevel", 12, newSVnv((double)*f),0);
OUTPUT:
	RETVAL

int
GetPowerSource(ghash)
HvObject *ghash;
PREINIT:
GSM_PowerSource *source;
CODE:
        source = (GSM_PowerSource *)malloc(sizeof(GSM_PowerSource));
        RETVAL = GSM->GetPowerSource(source);
        if(RETVAL == GE_NONE)
        {
          if(*source == GPS_ACDC)
            hv_store(ghash, "PowerSource", 11, newSVpv(strdup("ACDC"),0), 0);
	  else
	    hv_store(ghash, "PowerSource", 11, newSVpv(strdup("Battery"),0), 0);
	}
OUTPUT:
	RETVAL

int
EnterPin(ghash, pin)
HvObject *ghash;
char *pin;
CODE:
        RETVAL = GSM->EnterPin(pin);
OUTPUT:
	RETVAL


int
GetIMEI(ghash)
HvObject *ghash;
PREINIT:
char imei[20];
CODE:
        RETVAL = GSM->GetIMEI(imei);
        if(RETVAL == GE_NONE)
	  hv_store(ghash,"IMEI", 4, newSVpv(imei,0),0);
OUTPUT:
	RETVAL

int 
GetRevision(ghash)
HvObject *ghash;
PREINIT:
char revision[20];
CODE:
        RETVAL = GSM->GetRevision(revision);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "Revision", 8, newSVpv(revision, 0), 0);
OUTPUT:
	RETVAL

int 
GetModel(ghash)
HvObject *ghash;
PREINIT:
char model[8];
CODE:
        RETVAL = GSM->GetModel(model);
        if(RETVAL == GE_NONE)
          hv_store(ghash, "Model", 5, newSVpv(model, 0), 0);
OUTPUT:
	RETVAL

int
GetDateTime(ghash)
HvObject *ghash;
PREINIT:
Perl_GSM_DateTime *date_time;
struct tm t;
CODE:   
        date_time = (Perl_GSM_DateTime *)malloc(sizeof(Perl_GSM_DateTime));
        RETVAL = GSM->GetDateTime(date_time);
        if(RETVAL == GE_NONE)
          GSMDATE_TO_TM(date_time,t,ghash);
OUTPUT:
	RETVAL

int
SetDateTime(ghash,datetime)
HvObject *ghash;
int datetime;
PREINIT:
Perl_GSM_DateTime *dtime;
struct tm *t;
CODE:
        t = (struct tm *)malloc(sizeof(struct tm));
        t = localtime(&datetime);
        dtime = (Perl_GSM_DateTime *)malloc(sizeof(Perl_GSM_DateTime));
        dtime->Year = t->tm_year + 1900;
        dtime->Month = t->tm_mon +1;
        dtime->Day = t->tm_mday;
        dtime->Hour = t->tm_hour;
        dtime->Minute = t->tm_min;
        dtime->Second = t->tm_sec;
        RETVAL = GSM->SetDateTime(dtime);
OUTPUT:
	RETVAL

int
GetAlarm(ghash, alarm_number)
HvObject *ghash;
int alarm_number;
PREINIT:
Perl_GSM_DateTime *date_time;
struct tm t;
HvObject *ahash;
AvObject *aarray;
SV **ssv;
CODE:
        ssv = hv_fetch(ghash, "Alarms", 6, 0);
        if(ssv == NULL)
          aarray=newAV();
	else
        {
	  int j;
	  aarray=(AV *) SvRV(*ssv);
	  j=av_len(aarray);
	  while(j-->1)
	    av_pop(aarray);
	}

        date_time = (Perl_GSM_DateTime *)malloc(sizeof(Perl_GSM_DateTime));
        RETVAL = GSM->GetAlarm(alarm_number,date_time);
        if(RETVAL == GE_NONE)
        {
	  ahash=newHV();
          GSMDATE_TO_TM(date_time,t,ahash);
	  hv_store(ahash, "Enabled", 7, newSViv(date_time->AlarmEnabled),0);
	  av_store(aarray, alarm_number, newRV_inc((SV *)ahash));
	  if(ssv == NULL)
	    hv_store(ghash,"Alarms",6, newRV_inc((SV *)aarray),0);
	}
OUTPUT:
	RETVAL

int
SetAlarm(ghash, index, datetime)
HvObject *ghash;
int index;
int datetime;
PREINIT:
struct tm *t;
Perl_GSM_DateTime *date_time;
CODE:
        t = (struct tm *)malloc(sizeof(struct tm));
        t = localtime(&datetime);
        date_time = (Perl_GSM_DateTime *)malloc(sizeof(Perl_GSM_DateTime));
        date_time->Year = t->tm_year + 1900;
        date_time->Month = t->tm_mon +1;
        date_time->Day = t->tm_mday;
        date_time->Hour = t->tm_hour;
        date_time->Minute = t->tm_min;
        date_time->Second = t->tm_sec;
        date_time->AlarmEnabled = 1;
        RETVAL = GSM->SetAlarm(index,date_time);
OUTPUT:
        RETVAL

int 
DialVoice(ghash, number)
HvObject *ghash;
char *number;
CODE:
        RETVAL=GSM->DialVoice(number);
OUTPUT:
        RETVAL

int
GetIncomingCallNr(ghash)
HvObject *ghash;
PREINIT:
char *number;
CODE:
        RETVAL = GSM->GetIncomingCallNr(number);
        if(RETVAL == GE_NONE)
          hv_store(ghash,"Number",6, newSVpv(number, 0),0);
OUTPUT:
	RETVAL

int
SendBitmap(ghash,NetworkCode, width, height, bitmap)
HvObject *ghash;
char *NetworkCode;
int width;
int height;
unsigned char *bitmap;
CODE:
        RETVAL = GSM->SendBitmap(NetworkCode, width, height, bitmap);
OUTPUT:
	RETVAL

int
GetNetworkInfo(ghash)
HvObject *ghash;
PREINIT:
GSM_NetworkInfo *ni;
HvObject *ahash;
CODE:
        ni = (GSM_NetworkInfo *)malloc(sizeof(GSM_NetworkInfo));
        RETVAL = GSM->GetNetworkInfo(ni);
        if(RETVAL == GE_NONE)
        {
	  ahash = newHV();
	  hv_store(ahash, "NetworkCode", 11, newSVpv(ni->NetworkCode,0), 0);
	  hv_store(ahash, "CellID", 6, newSVpv(ni->CellID,0), 0);
	  hv_store(ahash, "LAC", 3, newSVpv(ni->LAC,0), 0);
	  hv_store(ghash, "NetworkInfo", 11, newRV_inc((SV *)ahash),0);
	}
OUTPUT:
	RETVAL

void
finish()
     CODE:
           GSM->Terminate();

double
constant(name,arg)
	char *		name
	int		arg

